@@ -5,6 +5,7 @@ setClass(
          slots=c(
            pars = 'character',
            Nmcmc = 'integer',
+           accepts = 'integer',
            proposal = 'function',
            conv.rec = 'array',
            log.prior = 'numeric'
@@ -12,6 +13,7 @@ setClass(
          prototype=prototype(
            pars = character(0),
            Nmcmc = 0L,
+           accepts = 0L,
            proposal = function (...) stop("proposal not specified"),
            conv.rec=array(dim=c(0,0)),
            log.prior=numeric(0)
@@ -23,34 +25,37 @@ pmcmc.internal <- function (object, Nmcmc,
                             Np, tol, max.fail,
                             verbose,
                             .ndone = 0L,
+                            .accepts = 0L,
                             .prev.pfp = NULL, .prev.log.prior = NULL,
                             .getnativesymbolinfo = TRUE) {
 
   object <- as(object,"pomp")
   gnsi <- as.logical(.getnativesymbolinfo)
+  verbose <- as.logical(verbose)
   .ndone <- as.integer(.ndone)
-
+  .accepts <- as.integer(.accepts)
+  
   pompLoad(object)
 
   if (missing(start))
-    stop(sQuote("start")," must be specified",call.=FALSE)
+    stop(sQuote("start")," must be specified")
   if (length(start)==0)
-    stop(
-         sQuote("start")," must be specified if ",
-         sQuote("coef(object)")," is NULL"
-         )
+    stop(sQuote("start")," must be specified if ",
+         sQuote("coef(object)")," is NULL")
   if (is.null(names(start)))
-    stop("pmcmc error: ",sQuote("start")," must be a named vector",call.=FALSE)
+    stop(sQuote("pmcmc")," error: ",
+         sQuote("start")," must be a named vector",call.=FALSE)
 
   if (!is.function(proposal))
     stop(sQuote("proposal")," must be a function")
 
   ## test proposal distribution
-  theta <- try(proposal(start))
+  theta <- try(proposal(start,.n=0))
   if (inherits(theta,"try-error"))
-    stop("pmcmc error: error in proposal function",call.=FALSE)
+    stop(sQuote("pmcmc")," error: error in proposal function",call.=FALSE)
   if (is.null(names(theta)) || !is.numeric(theta))
-    stop("pmcmc error: ",sQuote("proposal")," must return a named numeric vector",call.=FALSE)
+    stop(sQuote("pmcmc")," error: ",sQuote("proposal"),
+         " must return a named numeric vector",call.=FALSE)
 
   ntimes <- length(time(object))
   if (missing(Np))
@@ -80,11 +85,9 @@ pmcmc.internal <- function (object, Nmcmc,
     stop("pmcmc error: ",sQuote("Nmcmc")," must be a positive integer",call.=FALSE)
 
   if (verbose) {
-    cat("performing",Nmcmc,"PMCMC iteration(s) using",Np,"particles\n")
+    cat("performing",Nmcmc,"PMCMC iteration(s) using",Np[1L],"particles\n")
   }
 
-  theta <- start
-
   conv.rec <- matrix(
                      data=NA,
                      nrow=Nmcmc+1,
@@ -106,70 +109,96 @@ pmcmc.internal <- function (object, Nmcmc,
                                 pred.mean=FALSE,
                                 pred.var=FALSE,
                                 filter.mean=TRUE,
+                                filter.traj=TRUE,
                                 save.states=FALSE,
                                 save.params=FALSE,
                                 .transform=FALSE,
-                                verbose=verbose,
+                                verbose=FALSE,
                                 .getnativesymbolinfo=gnsi
                                 ),
-               silent=FALSE
+               silent=TRUE
                )
     if (inherits(pfp,'try-error'))
-      stop("pmcmc error: error in ",sQuote("pfilter"),call.=FALSE)
+      stop("in ",sQuote("pmcmc"),": error in ",sQuote("pfilter"),
+           ": ",pfp,call.=FALSE)
     log.prior <- dprior(object,params=theta,log=TRUE,.getnativesymbolinfo=gnsi)
     gnsi <- FALSE
   } else { ## has been computed previously
     pfp <- .prev.pfp
     log.prior <- .prev.log.prior
+    pfp@filter.traj <- pfp@filter.traj[,.ndone,,drop=FALSE]
   }
   conv.rec[1,names(theta)] <- theta
   conv.rec[1,c(1,2,3)] <- c(pfp@loglik,log.prior,pfp@nfail)
+  
+  filt.t <- array(
+                  data=0,
+                  dim=replace(dim(pfp@filter.traj),2L,Nmcmc),
+                  dimnames=replace(dimnames(pfp@filter.traj),2L,
+                    list(as.character(seq_len(Nmcmc))))
+                  )
 
   for (n in seq_len(Nmcmc)) { # main loop
 
-    theta.prop <- proposal(theta)
+    theta.prop <- proposal(theta,.n=n+.ndone,.accepts=.accepts,
+                           verbose=verbose)
 
-    ## run the particle filter on the proposed new parameter values
-    pfp.prop <- try(
-                    pfilter.internal(
-                                     object=pfp,
-                                     params=theta.prop,
-                                     Np=Np,
-                                     tol=tol,
-                                     max.fail=max.fail,
-                                     pred.mean=FALSE,
-                                     pred.var=FALSE,
-                                     filter.mean=TRUE,
-                                     save.states=FALSE,
-                                     save.params=FALSE,
-                                     .transform=FALSE,
-                                     verbose=verbose,
-                                     .getnativesymbolinfo=gnsi
-                                     ),
-                    silent=FALSE
-                    )
-    if (inherits(pfp.prop,'try-error'))
-      stop("pmcmc error: error in ",sQuote("pfilter"),call.=FALSE)
-    log.prior.prop <- dprior(object,params=theta.prop,log=TRUE,.getnativesymbolinfo=gnsi)
-    gnsi <- FALSE
+    ## compute log prior
+    log.prior.prop <- dprior(object,params=theta.prop,log=TRUE,
+                             .getnativesymbolinfo=gnsi)
 
-    ## PMCMC update rule (OK because proposal is symmetric)
-    if (runif(1) < exp(pfp.prop@loglik+log.prior.prop-pfp@loglik-log.prior)) {
-      pfp <- pfp.prop
-      theta <- theta.prop
-      log.prior <- log.prior.prop
+    if (is.finite(log.prior.prop)) {
+      
+      ## run the particle filter on the proposed new parameter values
+      pfp.prop <- try(
+                      pfilter.internal(
+                                       object=pfp,
+                                       params=theta.prop,
+                                       Np=Np,
+                                       tol=tol,
+                                       max.fail=max.fail,
+                                       pred.mean=FALSE,
+                                       pred.var=FALSE,
+                                       filter.mean=TRUE,
+                                       filter.traj=TRUE,
+                                       save.states=FALSE,
+                                       save.params=FALSE,
+                                       .transform=FALSE,
+                                       verbose=FALSE,
+                                       .getnativesymbolinfo=gnsi
+                                       ),
+                      silent=TRUE
+                      )
+      if (inherits(pfp.prop,'try-error'))
+        stop("in ",sQuote("pmcmc"),": error in ",sQuote("pfilter"),
+             ": ",pfp.prop,call.=FALSE)
+      gnsi <- FALSE
+
+      ## PMCMC update rule (OK because proposal is symmetric)
+      alpha <- exp(pfp.prop@loglik+log.prior.prop-pfp@loglik-log.prior)
+      if (runif(1) < alpha) {
+        pfp <- pfp.prop
+        theta <- theta.prop
+        log.prior <- log.prior.prop
+        .accepts <- .accepts+1L
+      }
     }
 
+    ## add filtered trajectory to the store
+    filt.t[,n,] <- pfp@filter.traj[,1L,]
+
     ## store a record of this iteration
     conv.rec[n+1,names(theta)] <- theta
     conv.rec[n+1,c(1,2,3)] <- c(pfp@loglik,log.prior,pfp@nfail)
 
-    if (verbose) cat("PMCMC iteration ",n," of ",Nmcmc," completed\n")
+    if (verbose) cat("PMCMC iteration",n+.ndone,"of",Nmcmc+.ndone,
+                     "completed\nacceptance ratio:",
+                     round(.accepts/(n+.ndone),3),"\n")
 
   }
 
   pars <- apply(conv.rec,2,function(x)diff(range(x))>0)
-  pars <- names(pars[pars])
+  pars <- setdiff(names(pars[pars]),c("loglik","log.prior","nfail"))
 
   pompUnload(object)
 
@@ -179,11 +208,13 @@ pmcmc.internal <- function (object, Nmcmc,
       params=theta,
       pars=pars,
       Nmcmc=Nmcmc,
+      accepts=.accepts,
       proposal=proposal,
       Np=Np,
       tol=tol,
       conv.rec=conv.rec,
-      log.prior=log.prior
+      log.prior=log.prior, 
+      filter.traj=filt.t
       )
 }
 
@@ -191,34 +222,20 @@ setMethod(
           "pmcmc",
           signature=signature(object="pomp"),
           function (object, Nmcmc = 1,
-                    start, proposal, pars, rw.sd, Np,
-                    tol = 1e-17, max.fail = 0,
+                    start, proposal, Np,
+                    tol = 1e-17, max.fail = Inf,
                     verbose = getOption("verbose"),
                     ...) {
             
             if (missing(start)) start <- coef(object)
             if (missing(Np))
               stop("pmcmc error: ",sQuote("Np")," must be specified",call.=FALSE)
-              
+            
             if (missing(proposal)) proposal <- NULL
 
-            if (!missing(rw.sd)) {
-              warning("pmcmc warning: ",sQuote("rw.sd")," is a deprecated argument.",
-                      "Use ",sQuote("proposal")," instead.",call.=FALSE)
-              if (is.null(proposal)) {
-                proposal <- mvn.diag.rw(rw.sd=rw.sd)
-              } else {
-                warning("pmcmc warning: since ",sQuote("proposal"),
-                        " has been specified, ",sQuote("rw.sd")," is ignored.")
-              }
-            }
-
             if (is.null(proposal))
               stop("pmcmc error: ",sQuote("proposal")," must be specified",call.=FALSE)
 
-            if (!missing(pars))
-              warning("pmcmc warning: ",sQuote("pars")," is a deprecated argument and will be ignored.",call.=FALSE)
-
             pmcmc.internal(
                            object=object,
                            Nmcmc=Nmcmc,
@@ -256,7 +273,7 @@ setMethod(
           signature=signature(object="pmcmc"),
           function (object, Nmcmc,
                     start, proposal,
-                    Np, tol, max.fail = 0,
+                    Np, tol, max.fail = Inf,
                     verbose = getOption("verbose"),
                     ...) {
 
@@ -286,12 +303,14 @@ setMethod(
           function (object, Nmcmc = 1, ...) {
 
             ndone <- object@Nmcmc
+            accepts <- object@accepts
 
             obj <- pmcmc(
                          object=object,
                          Nmcmc=Nmcmc,
                          ...,
                          .ndone=ndone,
+                         .accepts=accepts,
                          .prev.pfp=as(object,"pfilterd.pomp"),
                          .prev.log.prior=object@log.prior
                          )
@@ -301,7 +320,15 @@ setMethod(
                                   obj@conv.rec[-1,]
                                   )
             names(dimnames(obj@conv.rec)) <- c("iteration","variable")
+            ft <- array(dim=replace(dim(obj@filter.traj),2L,ndone+Nmcmc),
+                        dimnames=replace(dimnames(obj@filter.traj),2L,
+                          list(seq_len(ndone+Nmcmc))))
+            ft[,seq_len(ndone),] <- object@filter.traj
+            ft[,ndone+seq_len(Nmcmc),] <- obj@filter.traj
+            obj@filter.traj <- ft
             obj@Nmcmc <- as.integer(ndone+Nmcmc)
+            obj@accepts <- as.integer(accepts+obj@accepts)
+
             obj
           }
           )
