@@ -2,10 +2,14 @@
 
 rmeasure.internal <- function (object, x, times, params,
                                .getnativesymbolinfo = TRUE, ...) {
-  .Call(do_rmeasure,object,x,times,params,.getnativesymbolinfo)
+  pompLoad(object)
+  rv <- .Call(do_rmeasure,object,x,times,params,.getnativesymbolinfo)
+  pompUnload(object)
+  rv
 }
 
-setMethod("rmeasure","pomp",
-          function (object, x, times, params, ...)
+setMethod("rmeasure",
+          signature=signature("pomp"),
+          definition=function (object, x, times, params, ...)
           rmeasure.internal(object=object,x=x,times=times,params=params,...)
           )
