@@ -20,7 +20,7 @@ static void pomp_acf_compute (double *acf, double *x, int n, int nvars, int *lag
 	ct++;
       }
     }
-    if (ct < 1) error("series %ld has no data",j);
+    if (ct < 1) error("series %ld has no data",j+1);
     xx /= ct;			// mean of x[j,]
     for (k = 0, p0 = p; k < n; p0 += nvars, k++)
       if (R_FINITE(*p0)) *p0 -= xx;
@@ -54,7 +54,7 @@ static void pomp_ccf_compute (double *ccf, double *x, double *y, int n, int *lag
       ct++;
     }
   }
-  if (ct < 1) error("series 'x' has no data");
+  if (ct < 1) error("series 1 has no data");
   xx /= ct;			// mean of x[j]
   for (k = 0, p = x; k < n; k++, p++)
     if (R_FINITE(*p)) *p -= xx;
@@ -66,7 +66,7 @@ static void pomp_ccf_compute (double *ccf, double *x, double *y, int n, int *lag
       ct++;
     }
   }
-  if (ct < 1) error("series 'y' has no data");
+  if (ct < 1) error("series 2 has no data");
   xx /= ct;			// mean of y[j]
   for (k = 0, p = y; k < n; k++, p++)
     if (R_FINITE(*p)) *p -= xx;
