@@ -97,7 +97,7 @@ void iterate_map_R (double *X, double *time, double *p,
 SEXP iterate_map (SEXP object, SEXP times, SEXP t0, SEXP x0, SEXP params, SEXP gnsi)
 {
   int nprotect = 0;
-  int mode = -1;
+  pompfunmode mode = undef;
   SEXP fn, args;
   SEXP X;
   SEXP Snames, Pnames, Cnames;
@@ -156,7 +156,7 @@ SEXP iterate_map (SEXP object, SEXP times, SEXP t0, SEXP x0, SEXP params, SEXP g
 
   // set up the computations
   switch (mode) {
-  case 0:                       // R function
+  case Rfun:                       // R function
     {
       int nprotect = 0;
       SEXP cvec, tvec, xvec, pvec;
@@ -190,7 +190,7 @@ SEXP iterate_map (SEXP object, SEXP times, SEXP t0, SEXP x0, SEXP params, SEXP g
       UNPROTECT(nprotect);
     }
     break;
-  case 1:                       // native skeleton
+  case native:                       // native skeleton
     {
       int nprotect = 0;
       int *sidx, *pidx, *cidx;
@@ -218,7 +218,7 @@ SEXP iterate_map (SEXP object, SEXP times, SEXP t0, SEXP x0, SEXP params, SEXP g
 
 static struct {
   struct {
-    int mode;
+    pompfunmode mode;
     SEXP object;
     SEXP params;
     lookup_table covar_table;
@@ -250,7 +250,7 @@ static struct {
 
 SEXP pomp_desolve_setup (SEXP object, SEXP x0, SEXP params, SEXP gnsi) {
   int nprotect = 0;
-  int mode = -1;
+  pompfunmode mode = undef;
   SEXP fn, args;
   SEXP Snames, Pnames, Cnames;
   SEXP pompfun;
@@ -290,7 +290,7 @@ SEXP pomp_desolve_setup (SEXP object, SEXP x0, SEXP params, SEXP gnsi) {
   PROTECT(Cnames = GET_COLNAMES(GET_DIMNAMES(GET_SLOT(object,install("covar"))))); nprotect++;
 
   switch (COMMON(mode)) {
-  case 0:			// R function
+  case Rfun:			// R function
     // arguments of the R function
     PROTECT(RFUN(tvec) = NEW_NUMERIC(1)); nprotect++;
     PROTECT(RFUN(xvec) = NEW_NUMERIC(nvars)); nprotect++;
@@ -330,7 +330,7 @@ SEXP pomp_desolve_setup (SEXP object, SEXP x0, SEXP params, SEXP gnsi) {
     R_PreserveObject(RFUN(cvec));
 
     break;
-  case 1:			// native code
+  case native:			// native code
     // set aside userdata
     NAT(args) = args;
     // construct index vectors
@@ -362,14 +362,14 @@ SEXP pomp_desolve_setup (SEXP object, SEXP x0, SEXP params, SEXP gnsi) {
 void pomp_vf_eval (int *neq, double *t, double *y, double *ydot, double *yout, int *ip) 
 {
   switch (COMMON(mode)) {
-  case 0:			// R function
+  case Rfun:			// R function
     eval_skeleton_R(ydot,t,y,REAL(COMMON(params)),
 		    RFUN(fcall),RFUN(rho),RFUN(Snames),
 		    REAL(RFUN(tvec)),REAL(RFUN(xvec)),REAL(RFUN(pvec)),REAL(RFUN(cvec)),
 		    COMMON(nvars),COMMON(npars),1,COMMON(nreps),COMMON(nreps),COMMON(nreps),
 		    &COMMON(covar_table));
     break;
-  case 1:			// native code
+  case native:			// native code
     eval_skeleton_native(ydot,t,y,REAL(COMMON(params)),
 			 COMMON(nvars),COMMON(npars),COMMON(ncovars),1,
 			 COMMON(nreps),COMMON(nreps),COMMON(nreps),
@@ -392,7 +392,7 @@ void pomp_desolve_takedown (void) {
   COMMON(ncovars) = 0;
   COMMON(nreps) = 0;
   switch (COMMON(mode)) {
-  case 0:			// R function
+  case Rfun:			// R function
     R_ReleaseObject(RFUN(fcall));
     R_ReleaseObject(RFUN(rho));
     R_ReleaseObject(RFUN(Snames));
@@ -408,7 +408,7 @@ void pomp_desolve_takedown (void) {
     RFUN(pvec) = R_NilValue;
     RFUN(cvec) = R_NilValue;
     break;
-  case 1:			// native code
+  case native:			// native code
     NAT(fun) = 0;
     R_ReleaseObject(NAT(args));
     R_ReleaseObject(NAT(sindex));
